<?php

namespace Extension\KonnektiveUtilPack;

use Application\Config;
use Application\Request;
use Application\Response;
use Application\CrmResponse;
use Application\CrmPayload;
use Application\Lang;
use Application\Session;
use Application\Extension;

class PayOn extends Common
{
    public $stepToAttempt;

    public function __construct()
    {
        $this->stepToAttempt     = (int) Session::get('steps.current.id');     
    }

    public function updateCrmPayload()
    {  
        $emerc3ds = Config::extensionsConfig(
            'KonnektiveUtilPack.pay_on'
        );

        if (
            Request::attributes()->get('action') === 'prospect' || empty($emerc3ds)
        ) {
            return;
        }

        $offerUrl = Request::getOfferUrl();

        $payload['redirectsTo']      = $offerUrl . 'extensions/KonnektiveUtilPack/PayOnPostResponse.php';
        $payload['errorRedirectsTo'] = $offerUrl . AJAX_PATH . 'extensions/konnektiveutilpack/payon-response-error';
        CrmPayload::update($payload);
    }

    public function captureCrmResponse()
    {  
        $emerc3ds = Config::extensionsConfig(
            'KonnektiveUtilPack.pay_on'
        );

        if (
            Request::attributes()->get('action') === 'prospect' || empty($emerc3ds)
        ) {
            return;
        }

        $response = CrmResponse::all();
        
        if (!$response['success'] && !empty($response['isMerchantRedirection'])) {
            Response::send(array(
                'success'         => true,
                'skipQueryParams' => true,
                'redirect'        => CrmResponse::get('redirect'),
            ));
        }
    }

    public function payonResponse()
    {
        $response     = Request::query()->all();
        $instancePath = Request::getInstancePath();

        if (!empty($response['orderData'])) {
            $orderData = json_decode($response['orderData'], true);
            if ($orderData['orderStatus'] == 'COMPLETE') {
                Session::set(
                    sprintf('steps.%d.orderId', $this->stepToAttempt), $orderData['orderId']
                );
                Session::set(
                    sprintf('steps.%d.customerId', $this->stepToAttempt), $orderData['customerId']
                );
                Session::set('queryParams.order_id', $orderData['orderId']);
                Session::set('queryParams.customer_id', $orderData['customerId']);
                $queryParams = Session::get('queryParams');
                $url         = rtrim($instancePath, '/') . '/' . Session::get('steps.next.link');
                
//                CrmResponse::replace(
//                    array('success' => true, 'orderId' => $orderData['orderId'], 'customerId' => $orderData['customerId'])
//                );
//                
//                $extension = Extension::getInstance();                
//                $extension->performEventActions('afterAnyCrmRequest');
                
                header(sprintf('Location: %s?%s', $url, http_build_query($queryParams)));
                exit;
            }
        }

        $errorMessage = Lang::get('exceptions.generic_error');

        foreach ($response as $key => $value) {
            if (preg_match("/errorMsg=?.*/i", $value, $match)) {
                $errorMessageParts = explode('=', $match[0]);
                if (!empty($errorMessageParts[1])) {
                    $errorMessage = $errorMessageParts[1];
                    break;
                }
            }
        }

        $queryParams          = Session::get('queryParams');
        $queryParams['error'] = $errorMessage;
        $url                  = rtrim($instancePath, '/') . '/' . Session::get('steps.current.link');
        header(sprintf('Location: %s?%s', $url, http_build_query($queryParams)));
        exit;

    }

    public function payonResponseError()
    {
        $response     = Request::query()->all();
        $instancePath = Request::getInstancePath();
        
        if(!empty($response['errorMsg'])) {
            $errorMessage = $response['errorMsg'];
        } else {
            $errorMessage = Lang::get('exceptions.generic_error');
        }

        $queryParams          = Session::get('queryParams');
        $queryParams['error'] = $errorMessage;
        $url                  = rtrim($instancePath, '/') . '/' . Session::get('steps.current.link');
        header(sprintf('Location: %s?%s', $url, http_build_query($queryParams)));
        exit;

    }

}

