<?php

namespace Extension\TrafficLoadBalancerNew;

use Application\Config;
use Application\Http;
use Application\Registry;
use Application\Request;
use Application\Session;
use Lazer\Classes\Database;
use Application\CrmResponse;

class Settings
{

    private function __construct()
    {
        return;
    }

    public static $possibleCards = array('visa', 'master', 'discover', 'amex', 'diners', 'jcb');

    public static function getLocal()
    {
        $percentage = array(1 => 0, 2 => 0);
        $cardPercentage = array();
        if(!empty(Config::extensionsConfig('TrafficLoadBalancerNew.enable_default_settings'))){
            Scrapper::$scrapSpecificScrapMethod = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.scrapping_method');
            $percentage[1] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step1');
            $percentage[2] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step2');
            $percentage[3] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step3');
            $percentage[4] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step4');
            $percentage[5] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step5');
        }

        if (!empty(Config::extensionsConfig('TrafficLoadBalancerNew.enable_card_scrapper')))
        {
            $cardPercentage = self::localCardPercentagePrepare(Config::extensionsConfig('TrafficLoadBalancerNew.cardFilter'));
        }
        
        $affiliates = array_replace(
                self::initializeAffiliates(), Session::get('affiliates', array())
        );
        $affiliateSettings = Database::table('affiliates')->findAll()->asArray();
        $mappedAffiliates = self::mapAffiliateKeys($affiliates);
        $selectedAffiliates = self::initializeAffiliates();       
        $enableAdvancedAffiliateLogic = Config::extensionsConfig('TrafficLoadBalancerNew.enable_affiliate_orderfilter');

        foreach ($affiliateSettings as $affiliateSetting)
        { 
            $matched = true;$affiliateKey = '';
            foreach ($mappedAffiliates as $key => $value)
            {
                if ($affiliateSetting[$key] !== $value)
                {
                    $matched = false;
                    break;
                }
                 $affiliateKey = $value;
            }
            
            if ($matched)
            {
                $affiliatePercentage = self::getAffiliatePercenatge($affiliateSetting);
               
                if(!empty(Config::extensionsConfig('TrafficLoadBalancerNew.enable_affiliate_orderfilter'))){
                    Scrapper::$scrapSpecificScrapMethod = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.scrapping_method');
                    $percentage[1] = empty($affiliatePercentage['step1']) ? 0 : (int) $affiliatePercentage['step1'];
                    $percentage[2] = empty($affiliatePercentage['step2']) ? 0 : (int) $affiliatePercentage['step2'];
                    $percentage[3] = empty($affiliatePercentage['step3']) ? 0 : (int) $affiliatePercentage['step3'];
                    $percentage[4] = empty($affiliatePercentage['step4']) ? 0 : (int) $affiliatePercentage['step4'];
                    $percentage[5] = empty($affiliatePercentage['step5']) ? 0 : (int) $affiliatePercentage['step5'];
                }
               
               
                $selectedAffiliates = $affiliates;
                break;
            }
        }
        
        
        if (count(array_filter($selectedAffiliates)) == 0 && $enableAdvancedAffiliateLogic)
        {
            $updatedAffilitesLogic = self::advancedAffilitesLogic();
            $percentage = $updatedAffilitesLogic['percentage'];
            $selectedAffiliates = $updatedAffilitesLogic['affiliates'];
            $cardPercentage = $updatedAffilitesLogic['cardPercentage'];
        }

        if (!empty($selectedAffiliates))
        {
            $subaffiliatePost = Config::extensionsConfig('TrafficLoadBalancerNew.subaffiliate_post');
            if (!$subaffiliatePost)
            {
                unset($selectedAffiliates['c1']);
            }
            
            unset($selectedAffiliates['c2']);
            unset($selectedAffiliates['c3']);
            unset($selectedAffiliates['c4']);
            unset($selectedAffiliates['c5']);
            unset($selectedAffiliates['sId']);
            unset($selectedAffiliates['opt']);
            unset($selectedAffiliates['click_id']);
            unset($selectedAffiliates['clickId']);
        }

        return array(
            'percentage' => $percentage, 'affiliates' => $selectedAffiliates,
            'card_details' => empty($cardPercentage) ? null : $cardPercentage
        );
    }
    
    private static function getAffiliatePercenatge($affiliateSettings){
        $affiliateScrapperSettings = Config::extensionsConfig('TrafficLoadBalancerNew.affiliateFilter');
        if(empty($affiliateScrapperSettings))
            return false;
        $percentage = array();
        foreach ($affiliateScrapperSettings as $val){
            if($val['affiliate'] == $affiliateSettings['id']){
                $percentage['step1'] = $val['step1'];
                $percentage['step2'] = $val['step2'];
                $percentage['step3'] = $val['step3'];
                $percentage['step4'] = $val['step4'];
                $percentage['step5'] = $val['step5'];
            }
        }
        return empty($percentage) ? false : $percentage;
    }

    private static function localCardPercentagePrepare($param)
    {
        //print_r($param);die;
        $cardPercentage = array();
        foreach (self::$possibleCards as $val)
        {
            //$cardPercentage[$val][0] = 0;
            foreach ($param as $value)
            {
                if($val == $value['card_type']){
                    //unset($cardPercentage[$val]);
                    $cardPercentage[$val][$value['card_filter_config']] = 
                             $value['card_percentage'];
                    
                }
            }
           /* $cardKeyStep1 = "card_" . $val . "_scrap1";
            $cardKeyStep2 = "card_" . $val . "_scrap2";
            $cardPercentage[$val][1] = empty($param[$cardKeyStep1]) ? 0 :
                    (int) $param[$cardKeyStep1];
            $cardPercentage[$val][2] = empty($param[$cardKeyStep2]) ? 0 :
                    (int) $param[$cardKeyStep2];*/

            $cardPercentage[$val]['details']['scrappedCount'] = 0;
            $cardPercentage[$val]['details']['hitsCount'] = 0;
            $cardPercentage[$val]['details']['scrapped'] = 0;
            $cardPercentage[$val]['details']['hits'] = 0;
        }

        return $cardPercentage;
    }

    private static function advancedAffilitesLogic()
    {
        $percentage = array(1 => 0, 2 => 0, 3 => 0, 4 => 0, 5 => 0);
        $cardPercentage = array();
        if(!empty(Config::extensionsConfig('TrafficLoadBalancerNew.enable_default_settings'))){
            Scrapper::$scrapSpecificScrapMethod = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.scrapping_method');
            $percentage[1] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step1');
            $percentage[2] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step2');
            $percentage[3] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step3');
            $percentage[4] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step4');
            $percentage[5] = (int) Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.step5');
        }
        
       
        $affiliates = array_replace(
                self::initializeAffiliates(), Session::get('affiliates', array())
        );
        $affiliateSettings = Database::table('affiliates')->findAll()->asArray();
        $mappedAffiliates = self::mapAffiliateKeys($affiliates);
        $selectedAffiliates = self::initializeAffiliates();
        $matchedArr = array();
        $lastMatched = array();

        foreach ($affiliateSettings as $k => $affiliateSetting)
        {
            $matched = true;
            foreach ($mappedAffiliates as $key => $value)
            {
                if (!empty($affiliateSetting[$key]) && !empty($value) && $affiliateSetting[$key] == $value)
                {
                    $lastMatched[$key] = $affiliateSetting['id'];
                }
            }
        }

        if (array_key_exists('affid', $lastMatched) || array_key_exists('afid', $lastMatched))
        {
            $v = array_count_values($lastMatched);
            $maxs = array_keys($v, max($v));
            $matchKey = $maxs[0] - 1;
           
            
            $affiliatePercentage = self::getAffiliatePercenatge($affiliateSettings[$matchKey]);
             Scrapper::$scrapSpecificScrapMethod = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.scrapping_method');   
            $percentage[1] = empty($affiliatePercentage['step1']) ? 0 : (int) $affiliatePercentage['step1'];
            $percentage[2] = empty($affiliatePercentage['step2']) ? 0 : (int) $affiliatePercentage['step2'];
            $percentage[3] = empty($affiliatePercentage['step3']) ? 0 : (int) $affiliatePercentage['step3'];
            $percentage[4] = empty($affiliatePercentage['step4']) ? 0 : (int) $affiliatePercentage['step4'];
            $percentage[5] = empty($affiliatePercentage['step5']) ? 0 : (int) $affiliatePercentage['step5'];
            
            if (!empty($affiliateSettings[$matchKey]['enable_card_specific_filter']))
            {
                $cardPercentage = self::localCardPercentagePrepare($affiliateSettings[$matchKey]);
            }
            $selectedAffiliates = $affiliates;
        }

        return array(
            'percentage' => $percentage, 'affiliates' => $selectedAffiliates,
            'cardPercentage' => $cardPercentage
        );
    }

    public static function getRemote()
    {
        $campaignRouterEnable = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.enable_campaign_router');
        $offerUrl = sprintf('%s/', rtrim(Request::getOfferUrl(), '/'));
        $affiliates = self::mapAffiliateKeys(Session::get('affiliates', array()));
        
        $enableAffiliateMapping = Config::extensionsConfig('TrafficLoadBalancerNew.enable_affiliate_mapping');
        if ($enableAffiliateMapping)
        {
            $affiliateMappingConfiguration = Config::extensionsConfig('TrafficLoadBalancerNew.affiliates');
            foreach ($affiliateMappingConfiguration as $value)
            {
                if (array_key_exists($value['aff_param'], $affiliates))
                {
                    $requestParams = Session::get('queryParams.' . strtoupper($value['mapped_param']));
                    if (!empty($requestParams))
                    {
                        $affiliates[$value['aff_param']] = $requestParams . '.' . $affiliates[$value['aff_param']];
                    }
                }
            }
        }
        
        $gateWaySwitcherId = Config::settings('gateway_switcher_id');
        $queryParams = array(
            'offer_url' => $offerUrl,
            'conf_scrap_count' => 0,
        );

        if (!empty($affiliates))
        {
            $queryParams = array_replace($queryParams, $affiliates);
            $subaffiliatePost = Config::extensionsConfig('TrafficLoadBalancerNew.subaffiliate_post');
            if (!$subaffiliatePost)
            {
                unset($queryParams['c1']);
                unset($queryParams['c2']);
                unset($queryParams['c3']);
                unset($queryParams['c4']);
                unset($queryParams['c5']);
                unset($queryParams['sId']);
            }
        }
        $queryString = http_build_query($queryParams);
        
        if(!empty($campaignRouterEnable)) {
            $apiEndpoint = rtrim(Registry::system('systemConstants.UNIFY_CAMPAIGN_ROUTER_URL'), '/api');
            $routerAccessToken = Config::settings('router_access_token');
            // $hostName = Config::settings('host_name');
            // $hostName = str_replace(array("http://","https://","/"), "", $hostName);
            // if(!empty($hostName))
            // {
            //     $apiEndpoint = 'https://'. $hostName;
            // }
            $url = sprintf(
                '%s/campaign-router/insert-data/%s/?%s', $apiEndpoint, $routerAccessToken, $queryString
            );
        } else {
            $apiEndpoint = rtrim(Registry::system('systemConstants.NEW_PLATFORM_URL'), '/api');

            $hostName = Config::settings('host_name');
            $hostName = str_replace(array("http://","https://","/"), "", $hostName);
            if(!empty($hostName))
            {
                $apiEndpoint = 'https://'. $hostName;
            }
            else{
                $apiEndpoint = $apiEndpoint . '/api';
            }

            $url = sprintf(
                '%s/pixel-router/%s/?%s', $apiEndpoint, $gateWaySwitcherId, $queryString
            );
        }
       
        $response = self::getResponse($url); 
        
        if(!empty($campaignRouterEnable)) {
            $settings = empty($response['result']) ? array() : $response['result'];
        } else {
            $settings = empty($response['data']) ? array() : json_decode($response['data'], true);
        }
       
        $blacklistArray['emailBlacklist'] = array();
        $blacklistArray['cardSuffixBlacklist'] = array();
        $blacklistArray['cardPrefixBlacklist'] = array();
        $blacklistArray['ipBlacklist'] = array();
        
        foreach ($settings['black_list'] as $key => $value)
        {
            if($value['entity'] == 'email') {
                array_push($blacklistArray['emailBlacklist'], $value['value']);
            }
            if($value['entity'] == 'card_suffix') {
                array_push($blacklistArray['cardSuffixBlacklist'], $value['value']);
            }
            if($value['entity'] == 'card_prefix') {
                array_push($blacklistArray['cardPrefixBlacklist'], $value['value']);
            }
            if($value['entity'] == 'ip_addr') {
                array_push($blacklistArray['ipBlacklist'], $value['value']);
            }
        }
        
        Session::set('extensions.trafficLoadBalancer.orderFilterDetials.blackList',array_filter($blacklistArray));
        Session::set('extensions.trafficLoadBalancer.orderFilterDetials.binList',array_filter($settings['action_on_bin']));
        if(Session::get('steps.current.id') == 1)
        {
            Session::set('extensions.trafficLoadBalancer.orderFilterDetials.pixelManagerId',$settings['pixel_manager_id']);
            Session::set('extensions.trafficLoadBalancer.orderFilterDetials.engine',$settings['engine']);
        }
        Session::set('extensions.trafficLoadBalancer.orderFilterDetials.disableTestRouting',$settings['disable_test_routing']);
        
        $percentage = array(
            1 => empty($settings['step1_route_percentage']) ? 0 : (int) $settings['step1_route_percentage'],
            2 => empty($settings['upsell_route_percentage']) ? 0 : (int) $settings['upsell_route_percentage'],
        );
        $uniqueId = $settings['unique_id'];

        $affiliates = self::initializeAffiliates();
        
        $prepaid = !empty($settings['prepaid_check']);
        
        Session::set('extensions.trafficLoadBalancer.orderFilterDetials.uniqueIds.unique_id', $settings['unique_id']);        
        
        if (!empty($settings['network']))
        {
            if (
                    !empty($settings['network']['key']) &&
                    !empty($settings['network']['network_unique_id'])
            )
            {
                if (strtolower($settings['network']['key']) === 'nid')
                {
                    $affiliates['nId'] = $settings['network']['network_unique_id'];
                    Session::set('extensions.trafficLoadBalancer.orderFilterDetials.uniqueIds.network_unique_id', $settings['network']['network_unique_id']);
                }
            }
            $percentage[1] = (int) $settings['network']['step1_route_percentage'];
            $percentage[2] = (int) $settings['network']['upsell_route_percentage'];
            $prepaid = !empty($settings['affiliate']['prepaid_check']);
        }

        if (!empty($settings['affiliate']))
        {
            if (
                    !empty($settings['affiliate']['key']) &&
                    !empty($settings['affiliate']['aff_unique_id'])
            )
            {
                $uniqueId = $settings['network']['aff_unique_id'];
                if (strtolower($settings['affiliate']['key']) === 'affid')
                {
                    $affiliates['affId'] = $settings['affiliate']['aff_unique_id'];
                }
                if (strtolower($settings['affiliate']['key']) === 'afid')
                {
                    $affiliates['afId'] = $settings['affiliate']['aff_unique_id'];
                }
            }
            $percentage[1] = (int) $settings['affiliate']['step1_route_percentage'];
            $percentage[2] = (int) $settings['affiliate']['upsell_route_percentage'];
            Session::set('extensions.trafficLoadBalancer.orderFilterDetials.uniqueIds.aff_unique_id', $settings['affiliate']['aff_unique_id']);
            $prepaid = !empty($settings['affiliate']['prepaid_check']);
        }

        if (!empty($settings['affiliate']) && !empty($settings['sub_affiliate']))
        {
            $deepestSubAffiliate = end($settings['sub_affiliate']);
            $percentage[1] = (int) $deepestSubAffiliate['step1_route_percentage'];
            $percentage[2] = (int) $deepestSubAffiliate['upsell_route_percentage'];
            $uniqueId = $deepestSubAffiliate['network']['sub_unique_aff_id'];
            $prepaid = $deepestSubAffiliate['prepaid_check'];

            foreach ($settings['sub_affiliate'] as $subAffiliate)
            {
                if (
                        !empty($subAffiliate['key']) &&
                        !empty($subAffiliate['sub_unique_aff_id'])
                )
                {
                    if (strtolower($subAffiliate['key']) === 'sid')
                    {
                        $affiliates['sId'] = $subAffiliate['sub_unique_aff_id'];
                        continue;
                    }
                    if (strtolower($subAffiliate['key']) === 'click_id')
                    {
                        $affiliates['clickId'] = $subAffiliate['sub_unique_aff_id'];
                        continue;
                    }
                    $affiliates[$subAffiliate['key']] = $subAffiliate['sub_unique_aff_id'];
                    Session::set('extensions.trafficLoadBalancer.orderFilterDetials.uniqueIds.sub_unique_aff_id', $subAffiliate['sub_unique_aff_id']);
                }
            }
        }
       
        return array(
            'percentage' => $percentage,
            'affiliates' => $affiliates,
            'prepaid' => $prepaid,
            'card_details' => empty($cardPercentage) ? null : $cardPercentage,
            'engine' => $settings['engine']
        );
    }

    private static function cardPercentagePrepare($settings)
    {
        $cardPercentage = array();
        foreach (self::$possibleCards as $val)
        {

            $cardPercentage[$val][1] = empty($settings[$val]['step1_scrap_value']) ? 0 :
                    (int) $settings[$val]['step1_scrap_value'];
            $cardPercentage[$val][2] = empty($settings[$val]['upsell_scrap_value']) ? 0 :
                    (int) $settings[$val]['upsell_scrap_value'];

            $cardPercentage[$val]['details']['scrappedCount'] = 0;
            $cardPercentage[$val]['details']['hitsCount'] = 0;
            $cardPercentage[$val]['details']['scrapped'] = 0;
            $cardPercentage[$val]['details']['hits'] = 0;
        }

        return $cardPercentage;
    }

    private static function getResponse($url)
    {
        $response = Http::get($url, array(), array(CURLOPT_TIMEOUT => 5));
        return json_decode($response, true);
    }

    public static function initializeAffiliates()
    {
        return array(
            'nId' => '', 'afId' => '', 'affId' => '', 'sId' => '', 'c1' => '', 'c2' => '',
            'c3' => '', 'c4' => '', 'c5' => '', 'aId' => '', 'opt' => '',
            'clickId' => '',
        );
    }

    private static function mapAffiliateKeys($affiliates)
    {
        $mapping = array(
            'nId' => 'nid', 'afId' => 'afid', 'affId' => 'affid', 'sId' => 'sid', 'aId' => 'aid',
            'clickId' => 'click_id',
        );
        $newAffiliates = array();
        foreach ($affiliates as $key => $value)
        {
            if (array_key_exists($key, $mapping))
            {
                $newAffiliates[$mapping[$key]] = $value;
                continue;
            }
            $newAffiliates[$key] = $value;
        }
        return $newAffiliates;
    }
    
    public static function postRemoteData()
    {

        $offerUrl   = sprintf('%s/', rtrim(Request::getOfferUrl(), '/'));
        $affiliates = self::mapAffiliateKeys(Session::get('affiliates', array()));
        $campaignRouterEnable = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.enable_campaign_router');
        
        $enableAffiliateMapping = Config::extensionsConfig('TrafficLoadBalancerNew.enable_affiliate_mapping');
        if ($enableAffiliateMapping)
        {
            $affiliateMappingConfiguration = Config::extensionsConfig('TrafficLoadBalancerNew.affiliates');
            foreach ($affiliateMappingConfiguration as $value)
            {
                if (array_key_exists($value['aff_param'], $affiliates))
                {
                    $requestParams = Session::get('queryParams.' . strtoupper($value['mapped_param']));
                    if (!empty($requestParams))
                    {
                        $affiliates[$value['aff_param']] = $requestParams . '.' . $affiliates[$value['aff_param']];
                    }
                }
            }
        }
        
        $gateWaySwitcherId = Config::settings('gateway_switcher_id');
        $queryParams       = array(
            'offer_url'        => $offerUrl,
            'conf_scrap_count' => 0,
        );

        if (!empty($affiliates)) {
            $queryParams = array_replace($queryParams, $affiliates);
             $subaffiliatePost = Config::extensionsConfig('TrafficLoadBalancerNew.subaffiliate_post');            
            if(!$subaffiliatePost)
            {
                unset($queryParams['c1']);
                unset($queryParams['c2']);
                unset($queryParams['c3']);
                unset($queryParams['c4']);
                unset($queryParams['c5']);
                unset($queryParams['sId']);
            }
        }
        
        $currentStepId = Session::get('steps.current.id');        
        $queryParams['orderIds'] = array();
        
        for($i=1; $i<=$currentStepId; $i++) {
            if($currentStepId == $i) {
                $queryParams['orderIds'][$i] = CrmResponse::get('orderId');
            } else {
                $queryParams['orderIds'][$i] = session::get(sprintf('steps.%d.orderId',$i));
            }
        }

        $pixelManagerId = Session::get('extensions.trafficLoadBalancer.orderFilterDetials.pixelManagerId');
        $uniqueIds = Session::get('extensions.trafficLoadBalancer.orderFilterDetials.uniqueIds');
        $queryParams['orderIds'] = array_filter($queryParams['orderIds']);
        $queryParams['uniqueIds'] = array_filter($uniqueIds);
        $queryParams['uniqueIds']['pixelManagerId'] = $pixelManagerId;
        $queryParams['orderFilter'] = 'no';

        $cardDetails = Session::get('customer.cardNumber');
        $cardPrefix = substr(trim($cardDetails), 0, 6);
        $queryParams['bin'] = $cardPrefix;
        
        if(Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.enable_remote') || Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.enable_campaign_router')){
            $scrapperStepId    = $currentStepId > 1 ? 2 : 1;
        }else{
            $scrapperStepId = $currentStepId;
        }
        
        $currentLoadBalancer = Session::get(
                            sprintf('extensions.trafficLoadBalancer.%d', $scrapperStepId)
            );
        
        $isDisablePrepaidOrderFiler = Config::extensionsConfig('TrafficLoadBalancerNew.default_settings.disable_prepaid_orderfilter');
        
        if (
                    $currentLoadBalancer['scrapped'] && !$isDisablePrepaidOrderFiler && Session::has('steps.meta.isPrepaidFlow') ||
                    $currentLoadBalancer['scrapped'] && !Session::has('steps.meta.isPrepaidFlow')
        )
        {
            $queryParams['orderFilter'] = 'yes';
        }
            
        if(($currentStepId > 2) && Session::get('steps.meta.isScrapFlow')) 
        {
            $queryParams['orderFilter'] = 'yes';
        }
        
        $queryString = http_build_query($queryParams);
        
        if(!empty($campaignRouterEnable)) {
            $apiEndpoint = rtrim(Registry::system('systemConstants.UNIFY_CAMPAIGN_ROUTER_URL'), '/api');
            $routerAccessToken = Config::settings('router_access_token');
            // $hostName = Config::settings('host_name');
            // $hostName = str_replace(array("http://","https://","/"), "", $hostName);
            // if(!empty($hostName))
            // {
            //     $apiEndpoint = 'https://'. $hostName;
            // }
            
            if($currentStepId == 1) {
                $queryString2 = $queryString.'&convert=yes&sales=yes';
            } else {
                $queryString2 = $queryString.'&convert=yes';
            }

            $url = sprintf(
                '%s/campaign-router/insert-data/%s/?%s', $apiEndpoint, $routerAccessToken, $queryString2
            );
        } else {

            $apiEndpoint = rtrim(Registry::system('systemConstants.NEW_PLATFORM_URL'), '/api');
            $hostName = Config::settings('host_name');
            $hostName = str_replace(array("http://","https://","/"), "", $hostName);
            if(!empty($hostName))
            {
                $apiEndpoint = 'https://'. $hostName;
            }else{
                $apiEndpoint = $apiEndpoint . '/api';
            }

            if($currentStepId == 1) {
                $queryString2 = $queryString.'&convert=yes&sales=yes';
            } else {
                $queryString2 = $queryString.'&convert=yes';
            }

            $url = sprintf(
                '%s/pixel-router/%s/?%s', $apiEndpoint, $gateWaySwitcherId, $queryString2
            );
        
        }
        
        Session::set(sprintf('orderFilterUrl.%d', $currentStepId), $url);

        $response = Http::get($url, array(), array(CURLOPT_TIMEOUT => 5));
        
        Session::set(sprintf('orderFilterUrl.%d.response', $currentStepId), $response);
    }

}
